// ◇UTF-8
// cloudappdata: Google Drive Cloud App Data Tester
// Copyright (C)2013 GORRY.

package net.gorry.com.example.cloudappdata;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.gorry.com.example.cloudappdata.myGoogleCloudAppdata.myGoogleCloudAppdataListener;
import net.gorry.com.example.cloudappdata.R;

import com.google.android.gms.auth.GoogleAuthException;
import com.google.android.gms.auth.GoogleAuthUtil;
import com.google.android.gms.auth.UserRecoverableAuthException;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.googleapis.extensions.android.gms.auth.GoogleAccountCredential;
import com.google.api.client.json.gson.GsonFactory;
import com.google.api.services.drive.Drive;
import com.google.api.services.drive.Drive.Children;
import com.google.api.services.drive.DriveScopes;
import com.google.api.services.drive.model.About;
import com.google.api.services.drive.model.ChildList;
import com.google.api.services.drive.model.ChildReference;
import com.google.api.services.drive.model.File;
import com.google.api.services.drive.model.ParentReference;
import com.google.api.services.drive.model.Permission;

import android.net.Uri;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.accounts.AccountManager;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;

public class MainActivity extends Activity implements myGoogleCloudAppdataListener {
	private static final String TAG = "MainActivity";
	private static final boolean VV = true;//false;
	private static final boolean V = true;//false;
	private static final boolean D = true;//false;
	private static final boolean I = true;//false;

	private static final int REQUEST_ACCOUNT_PICKER = 1;
	private static final int REQUEST_AUTHORIZATION = 2;
	private static final String SAVE_FILE_NAME = "count.dat";
	
	private Activity me;
	
	private TextView mTextCount;
	private Button mButtonCountUp;
	private Button mButtonCountClear;
	private Button mButtonLogin;
	private Button mButtonLogout;
	private Button mButtonForgetAccount;
	private TextView mTextWait;
	private ProgressBar mProgressWait;

	private myGoogleCloudAppdata mGca;
	private byte[] mSaveData;
	
	/**
	 * onCreate()
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		if (V) Log.v(TAG, "onCreate(): start");
		super.onCreate(savedInstanceState);
		me = this;
		setContentView(R.layout.activitymain);

		// 画面コントロールの取得
		mTextCount = (TextView)findViewById(R.id.textCount);
		mButtonCountUp = (Button)findViewById(R.id.buttonCountUp);
		mButtonCountClear = (Button)findViewById(R.id.buttonCountClear);
		mButtonLogin = (Button)findViewById(R.id.buttonLogin);
		mButtonLogout = (Button)findViewById(R.id.buttonLogout);
		mButtonForgetAccount = (Button)findViewById(R.id.buttonForgetAccount);
		mTextWait = (TextView)findViewById(R.id.textWait);
		mProgressWait = (ProgressBar)findViewById(R.id.progressWait);

		// 画面コントロールの設定
		mButtonCountUp.setOnClickListener(mOnButtonClick);
		mButtonCountClear.setOnClickListener(mOnButtonClick);
		mButtonLogin.setOnClickListener(mOnButtonClick);
		mButtonLogout.setOnClickListener(mOnButtonClick);
		mButtonForgetAccount.setOnClickListener(mOnButtonClick);

		// 画面コントロールの初期表示
		mButtonCountUp.setEnabled(false);
		mButtonCountClear.setEnabled(false);
		mButtonLogout.setEnabled(false);
		printSaveCount();
		viewProgressWait(null);

		// GoogleCloudAppdataインスタンス作成
		mGca = new myGoogleCloudAppdata(me, this, REQUEST_ACCOUNT_PICKER, REQUEST_AUTHORIZATION);

		if (V) Log.v(TAG, "onCreate(): end");
	}

	/**
	 * onDestroy()
	 */
	@Override
	protected void onDestroy() {
		if (V) Log.v(TAG, "onDestroy(): start");
		super.onDestroy();
		mGca.logout();
		if (V) Log.v(TAG, "onDestroy(): start");
	}

	
	@Override
	protected void onActivityResult(final int requestCode, final int resultCode, final Intent data) {
		switch (requestCode) {
			case REQUEST_ACCOUNT_PICKER:
				Log.i(TAG, "onActivityResult(): REQUEST_ACCOUNT_PICKER: start");
				if (resultCode == RESULT_OK) {
					Log.i(TAG, "onActivityResult(): REQUEST_ACCOUNT_PICKER: RESULT_OK");
					mGca.onRequestAccountPickerActivity(data);
					Log.i(TAG, "onActivityResult(): REQUEST_ACCOUNT_PICKER: finish");
					return;
				}
				Log.i(TAG, "onActivityResult(): REQUEST_ACCOUNT_PICKER: end");
				break;

			case REQUEST_AUTHORIZATION:
				Log.i(TAG, "onActivityResult(): REQUEST_AUTHORIZATION: start");
				if (resultCode == RESULT_OK) {
					mGca.onRequestAccountAuthActivity();
					Log.i(TAG, "onActivityResult(): REQUEST_AUTHORIZATION: finish");
					return;
				}
				Log.i(TAG, "onActivityResult(): REQUEST_AUTHORIZATION: end");
				return;
		}
	}

	/**
	 * プログレスの表示
	 */
	private void viewProgressWait(final String str) {
		runOnUiThread(new Runnable(){ @Override public void run() {
			if ((str == null)||(str.length() == 0)) {
				mTextWait.setVisibility(View.INVISIBLE);
				mProgressWait.setVisibility(View.INVISIBLE);
			} else {
				mTextWait.setText(str);
				mTextWait.setVisibility(View.VISIBLE);
				mProgressWait.setVisibility(View.VISIBLE);
			}
		}});
	}

	/**
	 * ボタンクリックリスナ
	 */
	Button.OnClickListener mOnButtonClick = new Button.OnClickListener() {
		@Override
		public void onClick(View arg) {
			if (V) Log.v(TAG, "mOnButtonClick(): start");
			final Button btn = (Button)arg;
			new Thread(new Runnable() { @Override public void run() {
				if (btn == mButtonCountUp) {
					countup();
				}
				if (btn == mButtonCountClear) {
					clear();
				}
				if (btn == mButtonLogin) {
					login();
				}
				if (btn == mButtonLogout) {
					logout();
				}
				if (btn == mButtonForgetAccount) {
					signout();
				}
			}}).start();
			if (V) Log.v(TAG, "mOnButtonClick(): end");
		}
	};

	/**
	 * ログイン
	 */
	private synchronized void login() {
		viewProgressWait(getString(R.string.connecting));
		mGca.login();
	}
	
	/**
	 * ログアウト
	 */
	private synchronized void logout() {
		viewProgressWait(getString(R.string.disconnecting));
		mGca.logout();
	}
	
	/**
	 * サインアウト
	 */
	private synchronized void signout() {
		viewProgressWait(getString(R.string.disconnecting));
		mGca.signout();
	}
	
	/**
	 * セーブデータからのカウントを取り出す
	 */
	private int getSaveCount() {
		int count = ((mSaveData[0]&0xff)<<0) | ((mSaveData[1]&0xff)<<8) | ((mSaveData[2]&0xff)<<16) | ((mSaveData[3]&0xff)<<24);
		return count;
	}

	/**
	 * セーブデータへカウントを納める
	 * @param count カウント
	 */
	private void setSaveCount(int count) {
		mSaveData[0] = (byte)((count<< 0)&0xff);
		mSaveData[1] = (byte)((count<< 8)&0xff);
		mSaveData[2] = (byte)((count<<16)&0xff);
		mSaveData[3] = (byte)((count<<24)&0xff);
	}

	/**
	 * カウントの表示
	 */
	private void printSaveCount() {
		runOnUiThread(new Runnable(){ @Override public void run() {
			if (mSaveData == null) {
				mTextCount.setText("Not Logined");
			} else {
				int count = getSaveCount();
				mTextCount.setText(String.format("Count = %d",  count));
			}
		}});
	}
	
	/**
	 * カウントアップ
	 */
	private synchronized void countup() {
		if (V) Log.v(TAG, "countup(): start");
		if (mSaveData != null) {
			int count = getSaveCount();
			setSaveCount(count+1);
			if (V) Log.v(TAG, "countup(): run: start");
			try {
				viewProgressWait(getString(R.string.writing));
				mGca.writeFile(SAVE_FILE_NAME, mSaveData, null, null);
				printSaveCount();
				if (V) Log.v(TAG, "countup(): saved");
				viewProgressWait("");
			} catch (IOException e) {
				e.printStackTrace();
				viewProgressWait(getString(R.string.disconnecting));
				mGca.logout();
				return;
			}
		}
		if (V) Log.v(TAG, "countup(): end");
	}
	
	/**
	 * セーブデータのクリア
	 */
	private synchronized void clear() {
		if (V) Log.v(TAG, "clear(): start");
		try {
			viewProgressWait(getString(R.string.deleting));
			mGca.deleteFile(SAVE_FILE_NAME);
			mSaveData = new byte[4];
			printSaveCount();
			viewProgressWait("");
			if (V) Log.v(TAG, "countup(): cleared");
		} catch (IOException e) {
			e.printStackTrace();
			viewProgressWait(getString(R.string.disconnecting));
			mGca.logout();
			return;
		}
		if (V) Log.v(TAG, "clear(): end");
	}

	/**
	 * ログイン時の応答処理。非UIスレッドで実行される。
	 */
	@Override
	public void onLogined() {
		if (V) Log.v(TAG, "onLogined(): start");
		try {
			viewProgressWait(getString(R.string.reading));
			mSaveData = mGca.readFile(SAVE_FILE_NAME);
			if (mSaveData == null) {
				mSaveData = new byte[4];
			}
			printSaveCount();
			viewProgressWait("");
			runOnUiThread(new Runnable(){ @Override public void run() {
				mButtonCountUp.setEnabled(true);
				mButtonCountClear.setEnabled(true);
				mButtonLogout.setEnabled(true);
				mButtonLogin.setEnabled(false);
			}});
		} catch (IOException e) {
			e.printStackTrace();
			viewProgressWait(getString(R.string.disconnecting));
			mGca.logout();
			return;
		}
		if (V) Log.v(TAG, "onLogined(): end");
	}

	/**
	 * ログアウト時の応答処理。非UIスレッドで実行される。
	 */
	@Override
	public void onLogouted() {
		if (V) Log.v(TAG, "onLogouted(): start");
		mSaveData = null;
		viewProgressWait("");
		printSaveCount();
		runOnUiThread(new Runnable(){ public void run() {
			mButtonCountUp.setEnabled(false);
			mButtonCountClear.setEnabled(false);
			mButtonLogout.setEnabled(false);
			mButtonLogin.setEnabled(true);
		}});
		if (V) Log.v(TAG, "onLogouted(): end");
	}

	/**
	 * メニューを表示
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activitymain, menu);
		return true;
	}

	/**
	 * メニューを選択
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		boolean ret = true;
		switch (item.getItemId()) {
			default:
				ret = super.onOptionsItemSelected(item);
				break;
			case R.id.menu_about:
				showAbout();
				ret = true;
				break;
		}
		return ret;
	}

	
	/**
	 * Aboutを表示
	 */
	public void showAbout() {
		String versionName = null;
		final PackageManager pm = getPackageManager();
		try {
			PackageInfo info = pm.getPackageInfo(me.getPackageName(), 0);
			versionName = info.versionName;
		} catch (final NameNotFoundException e) {
			e.printStackTrace();
		}
		final AlertDialog.Builder bldr = new AlertDialog.Builder(me);
		bldr.setTitle(me.getString(R.string.app_name));
		bldr.setMessage("Version " + versionName + "\n" + me.getString(R.string.copyright));
		bldr.setPositiveButton(R.string.contact_uri_text, onContactClick);
		bldr.setIcon(R.drawable.ic_launcher);
		bldr.create().show();

	}

	/**
	 * [Contact]をクリック
	 */
	private final DialogInterface.OnClickListener onContactClick = new DialogInterface.OnClickListener() {
		@Override
		public void onClick(DialogInterface dialog, int item) {
			if (I) Log.i(TAG, "onContactClick: onClick()");
			String url = me.getString(R.string.contact_uri);
			String url2 = url;
			final Intent intent = new Intent();
			intent.setAction(Intent.ACTION_VIEW);
			intent.setData(Uri.parse(url2));
			me.startActivity(intent);
		}
	};



}

