// ◇UTF-8
// cloudappdata: Google Drive Cloud App Data Tester
// Copyright (C)2013 GORRY.

package net.gorry.com.example.cloudappdata;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.concurrent.CountDownLatch;

import android.app.Activity;

import android.accounts.AccountManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.preference.PreferenceManager;
import android.util.Log;

import com.google.android.gms.auth.GoogleAuthException;
import com.google.android.gms.auth.UserRecoverableAuthException;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.googleapis.extensions.android.gms.auth.GoogleAccountCredential;
import com.google.api.client.http.ByteArrayContent;
import com.google.api.client.http.GenericUrl;
import com.google.api.client.json.gson.GsonFactory;
import com.google.api.services.drive.Drive;
import com.google.api.services.drive.model.ParentReference;
import com.google.api.services.drive.model.About;


/**
 * 
 * Google DriveのAppdata領域を読み書きする
 *
 * @author gorry
 * 
 */
public class myGoogleCloudAppdata {
	static final String TAG = "myGoogleCloudAppdata";
	private static final boolean VV = true;//false;
	private static final boolean V = true;//false;
	private static final boolean D = true;//false;
	private static final boolean I = true;//false;

	private static final String DRIVESCOPES_APPDATA = "https://www.googleapis.com/auth/drive.appdata";
	private static final String SharedPreference_LastAccountName = "myGoogleCloudAppdata.lastAccountName";
	
	private Activity parent;
	private Drive mService;
	private GoogleAccountCredential mCredential;
	private int mActivityRequestCodeAccount;
	private int mActivityRequestCodeAuth;
	private int mActivityRequestCodeLogin;
	private myGoogleCloudAppdataListener mMyGoogleCloudAppdataListener;
	private boolean mLogined;
	
	/**
	 * GoogleCloudAppdataリスナ
	 */
	public interface myGoogleCloudAppdataListener {
		/**
		 * ログインが完了したときのリスナ。非UIスレッドで実行される。
		 */
		void onLogined();
		/**
		 * ログアウトが完了したときのリスナ。非UIスレッドで実行される。
		 */
		void onLogouted();
	}

	/**
	 * コンストラクタ
	 * @param activity アクティビティ
	 * @param listener リスナ
	 * @param requestCodeAccount アカウントピッカーからの応答を受け取るためのonActivityResult()リクエストコード
	 * @param requestCodeAuth 認証ダイアログからの応答を受け取るためのonActivityResult()リクエストコード
	 */
	public myGoogleCloudAppdata(final Activity activity, final myGoogleCloudAppdataListener listener, final int requestCodeAccount, final int requestCodeAuth) {
		parent = activity;
		mMyGoogleCloudAppdataListener = listener;
		mActivityRequestCodeAccount = requestCodeAccount;
		mActivityRequestCodeAuth = requestCodeAuth;
	}

	private GoogleAccountCredential getCredential() {
		return GoogleAccountCredential.usingOAuth2(parent, DRIVESCOPES_APPDATA);
	}
	
	private Drive getDriveService(GoogleAccountCredential credential) {
		return new Drive.Builder(AndroidHttp.newCompatibleTransport(), new GsonFactory(), credential).setApplicationName(parent.getPackageName()).build();
	}

	/**
	 * アカウントピッカーの起動
	 */
	public void startAccountPickerActivity() {
		parent.startActivityForResult(mCredential.newChooseAccountIntent(), mActivityRequestCodeAccount);
	}
	
	/**
	 * 認証ダイアログの起動
	 * @param intent 認証に必要な引数
	 */
	public void startAuthActivity(Intent intent) {
		parent.startActivityForResult(intent, mActivityRequestCodeAuth);
	}
	
	/**
	 * アカウントピッカーからの応答処理。onActivityResult()から呼び出す
	 * @param data onActivityResult()から受け取った引数
	 * @return 成功ならtrue
	 */
	public boolean onRequestAccountPickerActivity(final Intent data) {
		// アカウントピッカーからアカウントが返されたときの処理
		if (V) Log.v(TAG, "onRequestAccountPickerActivity(): start");

		if (data != null && data.getExtras() != null) {
			if (V) Log.v(TAG, "onRequestAccountPickerActivity(): have extra");
			String accountName = data.getStringExtra(AccountManager.KEY_ACCOUNT_NAME);
			if (D) Log.d(TAG, "onRequestAccountPickerActivity(): accountName=["+accountName+"]");
			if (accountName != null) {
				// このアカウント名を次回ログイン用に保存する
				final SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(parent);
				SharedPreferences.Editor editor = sp.edit();
				editor.putString(SharedPreference_LastAccountName, accountName);
				editor.commit();

				// Credentialが空なら確保する
				if (mCredential == null) {
					mCredential = getCredential();
				}

				// このアカウントでログインしてみる
				mCredential.setSelectedAccountName(accountName);
				mService = getDriveService(mCredential);
				onRequestAccountAuthActivity();
				if (V) Log.v(TAG, "onRequestAccountPickerActivity(): end: ret=true");
				return true;
			}
		}

		if (V) Log.v(TAG, "onRequestAccountPickerActivity(): end: ret=false");
		return false;
	}

	/**
	 * 認証ダイアログからの応答処理。onActivityResult()から呼び出す
	 * @return 成功ならtrue
	 */
	public boolean onRequestAccountAuthActivity() {
		if (V) Log.v(TAG, "onRequestAccountAuthActivity(): start");
		new Thread(new Runnable() { @Override public void run() {
			if (V) Log.v(TAG, "onRequestAccountAuthActivity(): run: start");
			try {
				String token = mCredential.getToken();
				if (D) Log.d(TAG, "onRequestAccountAuthActivity(): run: token=["+token+"]");
				onLogined();
				if (V) Log.v(TAG, "onRequestAccountAuthActivity(): run: finish");
				return;
			} catch (UserRecoverableAuthException e) {
				startAuthActivity(e.getIntent());
				if (V) Log.v(TAG, "onRequestAccountAuthActivity(): run: go auth");
				return;
			} catch (GoogleAuthException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			if (V) Log.v(TAG, "onRequestAccountAuthActivity(): run: end");
		}}).start();

		if (V) Log.v(TAG, "onRequestAccountAuthActivity(): end");
		return true;
	}

	/**
	 * ログイン開始処理。ログインそのものは別スレッドで実行される。
	 * @return 成功ならtrue
	 */
	public boolean login() {
		if (V) Log.v(TAG, "login(): start");
		new Thread(new Runnable() { @Override public void run() {
			if (V) Log.v(TAG, "login(): run: start");
			mCredential = getCredential();

			// 過去にログインしたアカウントがあれば、それでログインを試みる
			final SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(parent);
			String accountName = sp.getString(SharedPreference_LastAccountName, "");
			if (accountName.length() != 0) {
				if (D) Log.d(TAG, "login(): run: using last Account Name ["+accountName+"]");
				mCredential.setSelectedAccountName(accountName);
				try {
					mService = getDriveService(mCredential);
					String token = mCredential.getToken();
					if (D) Log.d(TAG, "login(): run: token=["+token+"]");
					onLogined();
					return;
				} catch (UserRecoverableAuthException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				} catch (GoogleAuthException e) {
					e.printStackTrace();
				}
			}

			// Credentialが空なら確保する
			if (mCredential == null) {
				mCredential = getCredential();
			}

			// ログインに使用するアカウントを選択する
			if (V) Log.v(TAG, "login(): run: select Account Name");
			startAccountPickerActivity();

			if (V) Log.v(TAG, "login(): run: end");
		}}).start();

		if (V) Log.v(TAG, "login(): end");
		return true;
	}
	
	/**
	 * ログイン完了時の処理
	 */
	private void onLogined() {
		mLogined = true;
		new Thread(new Runnable() { @Override public void run() {
			mMyGoogleCloudAppdataListener.onLogined();
		}}).start();
	}
	
	/**
	 * ログアウト処理
	 * @return 成功ならtrue
	 */
	public boolean logout() {
		if (V) Log.v(TAG, "logout(): start");

		mService = null;
		mCredential = null;
		mLogined = false;
		onLogouted();
		
		if (V) Log.v(TAG, "logout(): end");
		return true;
	}
	
	/**
	 * ログアウト完了時の処理
	 */
	private void onLogouted() {
		mMyGoogleCloudAppdataListener.onLogouted();
	}
	
	/**
	 * サインアウト処理。ログアウトして記憶しているアカウント情報も消す。
	 * @return 成功ならtrue
	 */
	public boolean signout() {
		if (V) Log.v(TAG, "signout(): start");

		logout();

		final SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(parent);
		SharedPreferences.Editor editor = sp.edit();
		editor.remove(SharedPreference_LastAccountName);
		editor.commit();

		if (V) Log.v(TAG, "signout(): end");
		return true;
	}
	
	/**
	 * ログインしているかどうかを返す。
	 * @return ログインしているならtrue
	 */
	public boolean isLogined() {
		return mLogined;
	}
	
	/**
	 * データの書き込み。
	 * UIスレッドでは実行不可、またこのメソッドの終了を非同期で待つことも不可。
	 * @param filename ファイル名
	 * @param writeData ファイル内容
	 * @param desc ファイルの説明。nullなら"appdata file"となる。新規作成時のみ有効。
	 * @param mime ファイルのMIME情報。nullなら"application/octet-stream"となる。新規作成時のみ有効。
	 * @return 成功ならtrue
	 * @throws IOException 書き込みに失敗した場合はIOExceptionが返る。
	 */
	public boolean writeFile(final String filename, final byte[] writeData, final String desc, final String mime) throws IOException {
		if (V) Log.v(TAG, "writeFile(): start: filename=["+filename+"]");
		if (Looper.myLooper() == Looper.getMainLooper()) {
			Log.e(TAG, "writeFile(): cannot run on UI Thread!");
			throw new IOException();
		}

		try {
			ByteArrayContent c = new ByteArrayContent(null, writeData);

			String query = "title contains '" + filename + "' and 'appdata' in parents";
			com.google.api.services.drive.model.FileList list = mService.files().list().setQ(query).execute();
			if (V) Log.v(TAG, "writeFile(): list=["+list+"]");
			if ((list != null) && (list.getItems().size() > 0)) {
				// ファイルがあればupdate
				com.google.api.services.drive.model.File file = list.getItems().get(0);
				if (D) Log.d(TAG, "writeFile(): file=["+file+"]");
				mService.files().update(file.getId(), file, c).execute();
				if (D) Log.d(TAG, "writeFile(): update filename=["+filename+"] "+writeData.length+"byte");
			} else {
				// ファイルがなければinsert
				com.google.api.services.drive.model.File body = new com.google.api.services.drive.model.File();
				body.setTitle(filename);
				body.setDescription((desc != null) ? desc : "appdata file");
				body.setMimeType((mime != null) ? mime : "application/octet-stream");
				body.setParents(Arrays.asList(new ParentReference().setId("appdata")));
				if (D) Log.d(TAG, "writeFile(): insert filename=["+filename+"] "+writeData.length+"byte: body=["+body+"]");
				mService.files().insert(body, c).execute();
			}
			if (V) Log.v(TAG, "writeFile(): success");
			return true;
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}

		// not reach
	}

	/**
	 * 入力ストリームを配列に読み込み
	 * @param fin 入力ストリーム
	 * @return 読み込んだデータ
	 * @throws IOException 読み込みに失敗した場合はIOExceptionが返る。
	 */
	private byte[] readAllFromStream(InputStream fin) throws IOException {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		byte[] buffer = new byte[1024*16];
		while (true) {
			int len = fin.read(buffer);
			if (len < 0) break;
			bout.write(buffer, 0, len);
		}
		byte[] ret = bout.toByteArray();
		return ret;
	}
	
	/**
	 * データの読み込み。
	 * UIスレッドでは実行不可、またこのメソッドの終了を非同期で待つことも不可。
	 * @param filename ファイル名
	 * @return 読み込んだデータ。ファイルがなければnullが返る。
	 * @throws IOException 読み込みに失敗した場合はIOExceptionが返る。
	 */
	public byte[] readFile(final String filename) throws IOException {
		if (V) Log.v(TAG, "readFile(): start: filename=["+filename+"]");
		if (Looper.myLooper() == Looper.getMainLooper()) {
			Log.e(TAG, "writeFile(): cannot run on UI Thread!");
			throw new IOException();
		}

		try {
			String query = "title contains '" + filename + "' and 'appdata' in parents";
			com.google.api.services.drive.model.FileList list = mService.files().list().setQ(query).execute();
			if (V) Log.v(TAG, "readFile(): list=["+list+"]");
			if ((list != null) && (list.getItems().size() > 0)) {
				// ファイルがあれば読み込む
				com.google.api.services.drive.model.File file = list.getItems().get(0);
				if (V) Log.v(TAG, "readFile(): file=["+file+"]");
				GenericUrl url = new GenericUrl(file.getDownloadUrl());
				if (V) Log.v(TAG, "readFile(): url=["+url.toString()+"]");
				com.google.api.client.http.HttpResponse res = mService.getRequestFactory().buildGetRequest(url).execute();
				InputStream fin = res.getContent();
				byte[] ret = readAllFromStream(fin);
				if (D) Log.d(TAG, "readFile(): read filename=["+filename+"] "+ret.length+"byte");
				return ret;
			}
			Log.w(TAG, "readFile(): no file");
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}

		// not reach
	}

	/**
	 * データの削除。
	 * UIスレッドでは実行不可、またこのメソッドの終了を非同期で待つことも不可。
	 * @param filename ファイル名
	 * @return 成功ならtrue
	 * @throws IOException 削除に失敗した場合はIOExceptionが返る。
	 */
	public boolean deleteFile(final String filename) throws IOException {
		if (V) Log.v(TAG, "deleteFile(): start: filename=["+filename+"]");
		if (Looper.myLooper() == Looper.getMainLooper()) {
			Log.e(TAG, "writeFile(): cannot run on UI Thread!");
			throw new IOException();
		}

		try {
			String query = "title contains '" + filename + "' and 'appdata' in parents";
			com.google.api.services.drive.model.FileList list = mService.files().list().setQ(query).execute();
			if (V) Log.v(TAG, "deleteFile(): list=["+list+"]");
			if ((list != null) && (list.getItems().size() > 0)) {
				com.google.api.services.drive.model.File file = list.getItems().get(0);
				if (D) Log.d(TAG, "deleteFile(): file=["+file+"]");
				mService.files().delete(file.getId()).execute();
				if (D) Log.d(TAG, "deleteFile(): delete filename=["+filename+"]");
				return true;
			}
			Log.w(TAG, "deleteFile(): run: no file");
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			throw e;
		}

		// not reach
	}
}

